// -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-

const AltTab = imports.ui.altTab;
const Meta = imports.gi.Meta;
const Shell = imports.gi.Shell;

var _originalInit;

var windowTracker;

const _init = function(apps, altTabPopup)
{
    // we will create our own apps list to pass to the original _init
    // apps are split into 2 if they have many windows across multiple workspaces

    let workspace = global.screen.get_active_workspace();

    // retrieve window list for all workspaces
    let allWindows = global.display.get_tab_list(Meta.TabList.NORMAL, null);

    let appsCurrent = [];
    let appsOther = [];

    for (let i = 0; i < allWindows.length; i++)
    {
        let windowApp = windowTracker.get_window_app(allWindows[i]);
        let j, appArray = appsCurrent;

        if (allWindows[i].get_workspace() != workspace)
            appArray = appsOther;

        for (j = 0; j < appArray.length; j++)
            if (appArray[j] == windowApp)
                break;

        if (j == appArray.length)
            appArray.push(windowApp);
    }

    // pass both apps arrays together to the original _init
    _originalInit.apply(this, [appsCurrent.concat(appsOther), altTabPopup]);

    let aIcons = this.icons;
    let appCount = appsCurrent.length; // number of apps on the current workspace

    // tidy up the cachedWindows on each AppIcon
    for (let i = 0; i < aIcons.length; i++)
    {
        if (aIcons[i].cachedWindows.length == 0) // support for Super+Tab Launcher extension
            continue;

        aIcons[i].actor.add_style_class_name(i < appCount ? 'alt-tab-app-mod' : 'alt-tab-app-other');

        for (let j = 0; j < aIcons[i].cachedWindows.length; j++)
            if ((i < appCount && aIcons[i].cachedWindows[j].get_workspace() != workspace) || (i >= appCount && aIcons[i].cachedWindows[j].get_workspace() == workspace))
                {
                    aIcons[i].cachedWindows.splice(j, 1);
                    j--; // don't accidentally skip any windows
                }

        if (aIcons[i].cachedWindows.length < 2)
            this._arrows[i].hide();
    }
};

function init()
{
    windowTracker = Shell.WindowTracker.get_default();
}

function enable()
{
    _originalInit = AltTab.AppSwitcher.prototype._init;
    AltTab.AppSwitcher.prototype._init = _init;
}

function disable()
{
    AltTab.AppSwitcher.prototype._init = _originalInit;
}
